# -*- coding: utf-8 -*-
################################################################################
#
# ROSA Crypto
#
# Copyright (c) 2016, LLC "NTC IT ROSA"
# License: BSD
# Authors:
#     Michl Voznesensky <m.voznesensky@rosalinux.ru>
#
# This file is a part of ROSA Crypto
#
# All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#  
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the LLC "NTC IT ROSA" nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

import sys
import os
import gettext
from PyQt4.QtCore import *
from PyQt4.QtGui import QApplication, QMainWindow, QWidget, \
						QGridLayout, QPushButton, QLabel, QLineEdit, \
						QTextEdit, QFileDialog, QAction, QActionGroup, \
						QSizePolicy, QHBoxLayout, QVBoxLayout, QFrame, \
						QComboBox, QCheckBox, QToolButton, QMenu, \
						QIcon, QKeySequence, QFont, QFontMetrics

#const size window app
max_width = 1024
min_width = 700
min_height = 320
max_height = 600
max_row_result = 45

prefix16 = os.path.dirname(__file__) + '/icon/16x16/' #directory icons
prefix32 = os.path.dirname(__file__) + '/icon/32x32/'
prefix48 = os.path.dirname(__file__) + '/icon/48x48/'

class MainWindow(QMainWindow):
	""" Main window """
	
	def __init__(self):
		""" Create main widget (window) """
		
		super(MainWindow, self).__init__()
		
		self.setFixedHeight(min_height)
		self.setMaximumWidth(max_width)
		
		self.main_widget = QWidget(self)
		self.welcome_text = QLabel(_("\n\n\nWelcome to program ROSA Crypto Tool !\n\nTo get started select the desired mode."))
		self.note_mess = QLabel()
		self.welcome_text.setAlignment(Qt.AlignHCenter)
		self.note_mess.setAlignment(Qt.AlignCenter)
		workspace = QVBoxLayout() #main workspace
		workspace.addWidget(self.welcome_text)
		workspace.addWidget(self.note_mess)
		self.main_widget.setLayout(workspace)
		self.setCentralWidget(self.main_widget)
		
		#Action
		#Can use QIcon.fromTheme, but this function work on Qt since 4.6.4
		
		self.opt_act = QAction(QIcon(prefix32 + 'options'), _("&Options"), self)
		self.opt_act.setToolTip(_("See also additional options"))
		
		self.open_act = QAction(QIcon(prefix16 + 'open'), _("&Choose File"), self)
		self.open_act.setShortcut(QKeySequence.Open)
		self.open_act.setCheckable(True)
		self.open_act.setToolTip(_("Choose file for sign or verify signature"))
		
		self.exit_act = QAction(QIcon(prefix16 + 'exit'), _("&Exit"), self)
		self.exit_act.setShortcut(QKeySequence("Ctrl+Q")) #Qt4 low 4.6.4 is not support QKeySequence.Quit
		
		self.sign_act = QAction(QIcon(prefix48 + 'sign'), _("&Sign\n file"), self)
		self.sign_act.setToolTip(_("Sign the selected file"))
		self.sign_act.setCheckable(True)
		
		self.check_sign_act = QAction(QIcon(prefix48 + 'verify_sign'), _("&Check\n signature"), self)
		self.check_sign_act.setToolTip(_("Verify a signed file"))
		self.check_sign_act.setCheckable(True)
		
		self.encrypt_act = QAction(QIcon(prefix48 + 'crypt'), _("&Encrypt"), self)
		self.encrypt_act.setToolTip(_("Encrypted file"))
		self.encrypt_act.setCheckable(True)
		
		self.decrypt_act = QAction(QIcon(prefix48 + 'crypt'), _("&Descrypt"), self)
		self.decrypt_act.setToolTip(_("Descrypted file"))
		self.decrypt_act.setCheckable(True)
		
		self.check_comp_act = QAction(QIcon(prefix16 + 'check'), _("Check component program"), self)
		
		self.help_act = QAction(QIcon(prefix16 + 'help'), _("&Help"), self)
		self.help_act.setShortcut(QKeySequence.HelpContents)
		
		self.about_act = QAction(QIcon(prefix16 + 'about'), _("&About ROSA Crypto Tool"), self)
		
		#StatusBar
		self.prog_icon_green = QIcon(prefix16 + 'green.svg').pixmap(16, 16)
		self.prog_icon_orange = QIcon(prefix16 + 'orange.svg').pixmap(16, 16)
		self.prog_icon_red = QIcon(prefix16 + 'red.svg').pixmap(16, 16)
		self.stat_prog_icon = QLabel()
		self.stat_prog_text = QLabel()
		self.statusBar().addWidget(self.stat_prog_icon)
		self.statusBar().addWidget(self.stat_prog_text)
		self.statusBar().setVisible(False)
		
		#Menu options
		opt_menu = QMenu(self)
		opt_menu.addAction(self.check_comp_act)
		opt_menu.addSeparator()
		opt_menu.addAction(self.about_act)
		opt_menu.addAction(self.help_act)
		opt_menu.addSeparator()
		opt_menu.addAction(self.exit_act)
		
		#ToolBar
		toolbar = self.addToolBar("Mode")
		toolbar.setMovable(False) #dockable panel
		toolbar.setContextMenuPolicy(Qt.PreventContextMenu) #disable context menu
		toolbar.setIconSize(QSize(32, 32))
		toolbar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
		
		options = QToolButton(self)
		options.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
		options.setPopupMode(QToolButton.InstantPopup)
		options.setDefaultAction(self.opt_act)
		options.setMenu(opt_menu)
		
		spacing_tb = QWidget()
		spacing_tb.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred)
		
		action_group = QActionGroup(self)
		action_group.addAction(self.sign_act)
		action_group.addAction(self.check_sign_act)
		action_group.addAction(self.encrypt_act)
		action_group.addAction(self.decrypt_act)
		
		toolbar.addActions(action_group.actions())
		map (lambda mode: toolbar.widgetForAction(mode).setFixedWidth(95), action_group.actions())
		toolbar.addWidget(spacing_tb) #spacing
		toolbar.addWidget(options)
		
		self.about_program = AboutWidget()
		
		self.sign = SignFrame(self.main_widget)
		self.check_sign = CheckFrame(self.main_widget)
		self.encrypt = EncryptFrame(self.main_widget)
		self.decrypt = DescryptFrame(self.main_widget)
		
		self.list_mode = [self.sign, self.check_sign, self.encrypt, self.decrypt]
		
		map(lambda mode: workspace.addWidget(mode), self.list_mode)
		
		toolbar.actionTriggered.connect(self.switchMode)
		self.about_act.triggered.connect(self.showAbout)
		self.list_off_token = [self.sign_act, self.decrypt_act, self.sign, self.decrypt]
		self.list_off_lic = [self.sign_act, self.sign]
		self.list_off_all = [self.sign_act, self.check_sign_act, self.encrypt_act, self.decrypt_act,\
							self.sign, self.check_sign, self.encrypt, self.decrypt]
		
		self.resize(min_width, min_height) #widht and hight window
		
		self.setWindowTitle(_("ROSA Crypto Tool"))
		self.setWindowIcon(QIcon(prefix48 + 'rosa-crypto-tool'))
		self.show()
	
	def showAbout(self):
		current_pos = self.about_program.geometry()
		current_pos.moveCenter(self.geometry().center())
		self.about_program.move(current_pos.topLeft())
		self.about_program.show()
	
	def switchMode(self, action):
		if self.welcome_text.isVisible():
			self.welcome_text.setVisible(False)
			self.note_mess.setVisible(False)
			self.statusBar().setVisible(True)
		
		if action == self.sign_act:
			self.__runMode(self.sign)
		elif action == self.check_sign_act:
			self.__runMode(self.check_sign)
		elif action == self.encrypt_act:
			self.__runMode(self.encrypt)
		elif action == self.decrypt_act:
			self.__runMode(self.decrypt)
	
	def __runMode(self, v_mode):
		""" Install visible target mode and resize window if needed """ 
		
		map(lambda mode: mode.setVisible(True) if mode == v_mode else mode.setVisible(False), self.list_mode)
		v_mode.checkWindow()

class AbstrFrame(QFrame):
	""" Basic widget for modes with CryptoPro
	Function showMode, resizeEvent, checkWindow
	responsible for right resize main window
	"""
	
	def __init__(self, parent = None):
		super (AbstrFrame, self).__init__(parent)
		
		self.parent = parent.parentWidget()
		self.setVisible(False)
		
		self.filter_file = _("All(*)")
		
		self.break_recursion = False
		self.elided_text = QFontMetrics(QFont())
		
		self.list_off_frame = [] #disable/enable components section
		
		#Labels
		file = QLabel(_("File:"))
		details = QLabel(_("Result:"))
		self.cert = QLabel(_("Certificate:"))
		self.mes_status_oper = QLabel()
		self.mes_status_oper.setFixedHeight(max_row_result)
		self.old_mes_status_oper = "" #nedeed for dynamic elided text
		self.mes_status_oper.setWordWrap(True)
		self.icon_status_oper = QLabel()
		
		self.icon_success = QIcon(prefix16 + 'success.svg').pixmap(22,22)
		self.icon_atten = QIcon(prefix16 + 'attention.svg').pixmap(22,22)
		self.icon_failed = QIcon(prefix16 + 'fail.svg').pixmap(22,22)
		
		#Fields Edit
		self.file_edit = QLineEdit(readOnly = True)
		self.result = QTextEdit(readOnly = True)
		self.result.setVisible(False)
		
		#Buttons
		self.btn_open = QPushButton(self.parent.open_act.icon(), _("Choose"))
		self.btn_open.setToolTip(self.parent.open_act.toolTip())
		
		self.btn_details = QPushButton(_("Details") + "  " u'\u25BC')
		self.btn_details.setToolTip(_("Show or hide more information"))
		self.btn_details.setEnabled(False)
		
		self.btn_details.clicked.connect(self.showMore)
		self.btn_open.clicked.connect(self.openFile)
		
		#Grid
		self.grid = QGridLayout()
		self.grid.setSpacing(5)
		
		self.grid.addWidget(file, 0, 0)
		self.grid.addWidget(self.file_edit, 0, 1)
		self.grid.addWidget(self.btn_open, 0, 2)
		
		self.grid.addWidget(details, 2, 0)
		hbox_status = QHBoxLayout()
		hbox_status.addWidget(self.icon_status_oper)
		hbox_status.addWidget(self.mes_status_oper, 1)
		self.grid.addLayout(hbox_status, 2, 1)
		self.grid.addWidget(self.result, 2, 1, -1, 1)
		self.grid.addWidget(self.btn_details, 2, 2)
		
		self.grid.setRowMinimumHeight(1, 95)
		self.grid.setRowMinimumHeight(2, max_row_result)
		self.grid.setColumnMinimumWidth(0, 80)
		self.grid.setColumnMinimumWidth(2, 180)
		self.grid.setRowStretch(3, 1)
		
		self.setLayout(self.grid)
	
	def showMore(self):
		""" Change height window and show/hide sub field """
		
		if self.result.isVisible():
			self.btn_details.setText(_("Details") + "  " u'\u25BC')
			self.result.setVisible(False)
			self.parent.setFixedHeight(min_height)
		else:
			self.btn_details.setText(_("Details") + "  " u'\u25B2')
			self.result.setVisible(True)
			self.parent.setFixedHeight(max_height)
	
	def openFile(self):
		""" Open file """
		
		self.path_name = QFileDialog.getOpenFileName(self, _("Choose file"), os.path.expanduser("~"), self.filter_file)
		
		if self.path_name:
			map(lambda item: item.setEnabled(True), self.list_off_frame)
			if self.result.isVisible():
				self.showMore()
			
			self.__parseDir(self.path_name) #create file_name and currently_dir
			self.file_edit.setText(self.file_name)
			#clear field
			if self.btn_details.isEnabled(): self.btn_details.setEnabled(False)
			self.icon_status_oper.clear()
			self.mes_status_oper.clear()
			self.result.clear()
	
	def resizeEvent(self, e):
		""" Check state window and resize him (max to min) """
		
		if not self.parent.isMaximized():
			if self.result.isVisible():
				self.parent.resize(QSize(self.parent.width(), max_height))
			else:
				self.parent.resize(QSize(self.parent.width(), min_height))
		
		self.mes_status_oper.setText(self.elided_text.elidedText(self.old_mes_status_oper, Qt.ElideRight, self.parent.width(), Qt.TextWordWrap))
	
	def checkWindow(self):
		""" Resize window for for each mode """
		
		if self.result.isVisible():
			self.parent.setFixedHeight(max_height)
		else:
			self.parent.setFixedHeight(min_height)
	
	def __parseDir(self, path):
		""" Share the current directory of the file and file name """
		
		#required type str to replace characters in the Latin alphabet if the original file name contains Cyrillic
		self.file_name = path[path.lastIndexOf("/")+1:] #search last entry "/" and make the cut
		self.currently_dir = unicode(path[:path.lastIndexOf("/")+1]) #search last entry "/" and make the cut

class SignFrame(AbstrFrame):
	""" Widget for signature """
	
	def __init__(self, parent = None):
		super (SignFrame, self).__init__(parent)
		
		self.btn_sign = QPushButton(self.parent.sign_act.icon(), _("Sign file"))
		self.btn_sign.setToolTip(self.parent.sign_act.toolTip())
		
		self.list_avail_cert = QComboBox(self)
		
		self.list_off_frame = [self.btn_sign, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_sign, 1, 2)

class CheckFrame(AbstrFrame):
	""" Widget for signature """
	
	def __init__(self, parent = None):
		super (CheckFrame, self).__init__(parent)
		
		self.filter_file = _("Signature file(*.sig)") + ";;" + self.filter_file
		
		self.btn_check_sign = QPushButton(self.parent.check_sign_act.icon(), _("Check signature"))
		self.btn_check_sign.setToolTip(self.parent.check_sign_act.toolTip())
		
		self.check_split = QCheckBox(_("Split file"))
		self.check_split.setToolTip(_("Separate the source file from the file signature"))
		self.check_install = QCheckBox(_("Install certificate of the signature file"))
		self.check_install.setToolTip(_("Install certificate in the local storage of the signature file"))
		
		vbox_check_sig = QVBoxLayout()
		vbox_check_sig.addWidget(self.check_split)
		vbox_check_sig.addWidget(self.check_install)
		
		self.list_off_frame = [self.btn_check_sign, self.check_split, self.check_install]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addLayout(vbox_check_sig, 1, 1)
		self.grid.addWidget(self.btn_check_sign, 1, 2)

class EncryptFrame(AbstrFrame):
	""" Widget for encrypted file """
	
	def __init__(self, parent = None):
		super (EncryptFrame, self).__init__(parent)
		
		self.list_avail_cert = QComboBox(self)
		self.btn_encrypt = QPushButton(self.parent.encrypt_act.icon(), self.parent.encrypt_act.text())
		self.btn_encrypt.setToolTip(self.parent.encrypt_act.toolTip())
		
		self.list_off_frame = [self.btn_encrypt, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_encrypt, 1, 2)

class DescryptFrame(AbstrFrame):
	""" Widget for encrypted file """
	
	def __init__(self, parent = None):
		super (DescryptFrame, self).__init__(parent)
		
		self.filter_file = _("Encrypted file(*.enc)") + ";;" + self.filter_file
		
		self.list_avail_cert = QComboBox(self)
		self.btn_decrypt = QPushButton(self.parent.decrypt_act.icon(), self.parent.decrypt_act.text())
		self.btn_decrypt.setToolTip(self.parent.decrypt_act.toolTip())
		
		self.list_off_frame = [self.btn_decrypt, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_decrypt, 1, 2)

class AboutWidget(QWidget):
	""" Widget about program """
	
	def __init__(self, parent = None):
		super (AboutWidget, self).__init__(parent)
		
		self.parent = parent
		
		rosa_logo = QIcon(prefix48 + 'rosa-crypto-tool.svg').pixmap(48,48)
		logoLabel = QLabel()
		logoLabel.setPixmap(rosa_logo)
		
		messages = QLabel()
		messages.setText(_("ROSA Crypto Tool - program for working with electronic digital signature\n\n"
						"Authors: Michl Voznesensky\n" +\
						"Version: beta 0.2.1\n" +\
						"License: BSD\n" +\
						"Copyright (c) 2016, LLC \"NTC IT ROSA\""))
		
		grid = QGridLayout()
		grid.setSpacing(5)
		grid.addWidget(logoLabel, 0, 0, Qt.AlignCenter)
		grid.addWidget(messages, 1, 0, Qt.AlignCenter)
		
		self.setLayout(grid)
		
		self.resize(250, 150) #widht and hight window
		
		self.setWindowTitle(_("About ROSA Crypto Tool"))
		self.setWindowIcon(QIcon(prefix16 + 'about'))

if __name__ == '__main__':
	
	print "You run the part program rosa crypto tool, please launch the main file"
	#for debugger UI
	
	#gettext.install('rosa-crypto-tool', unicode=True)
	#app = QApplication(sys.argv)
	#app_ui = MainWindow()
	#sys.exit(app.exec_())
	